#if UNITY_STANDALONE
using System;
using System.Collections;
using _Core;
using Newtonsoft.Json;
using Seayoo.ComboSDK.Windows;
using UnityEngine;
using UnityEngine.UI;
using Vuplex.WebView;

namespace _Combo
{
    [ViewPrefab("Prefabs/ComboWebView")]
    internal class WebView : View<WebView>
    {
        public string url = "";
        public CanvasRenderer navigationBar;
        public RectTransform comboWebviewRect;
        public LayoutElement contentLayout;
        public Text reloadText;
        public Button closeBtn;
        public Button backBtn;
        public Button forwardBtn;
        public Button refreshBtn;
        public Button reloadBtn;
        public RectTransform webviewRect;
        public CanvasRenderer errorPanel;
        public bool showNavigationBar;
        private CanvasWebViewPrefab canvasWebViewPrefab;
        private Action onCloaseAction;

        async void Start()
        {
            if(!showNavigationBar)
            {
                navigationBar.gameObject.SetActive(false);
            }
            else
            {
                float h = comboWebviewRect.rect.height;
                contentLayout.minHeight = (float)(h * 0.95);
            }
            canvasWebViewPrefab = CanvasWebViewPrefab.Instantiate();
            canvasWebViewPrefab.transform.SetParent(webviewRect, false);
            canvasWebViewPrefab.DragMode = DragMode.DragWithinPage;

            await canvasWebViewPrefab.WaitUntilInitialized();

            canvasWebViewPrefab.WebView.MessageEmitted += MessageEmitted;
            canvasWebViewPrefab.WebView.LoadProgressChanged += ProgressChanged;
            canvasWebViewPrefab.WebView.LoadUrl(url);

            var keyboardListener = FindObjectOfType<Vuplex.WebView.Internal.NativeKeyboardListener>();
            if (keyboardListener != null) {
                // 启用组件
                keyboardListener.enabled = true;
            }
            Input.imeCompositionMode = IMECompositionMode.On;
        }

        void Awake()
        {
            closeBtn.onClick.AddListener(()=>{
                CloseIME();
                onCloaseAction?.Invoke();
                Destroy();
            });
            backBtn.onClick.AddListener(() => canvasWebViewPrefab.WebView.GoBack());
            forwardBtn.onClick.AddListener(() => canvasWebViewPrefab.WebView.GoForward());
            refreshBtn.onClick.AddListener(() => canvasWebViewPrefab.WebView.Reload());
            reloadBtn.onClick.AddListener(() => canvasWebViewPrefab.WebView.Reload());
            backBtn.interactable = false;
            forwardBtn.interactable = false;
            reloadText.text = I18n.T("webview_reload_button");
        }

        void OnDestroy()
        {
            closeBtn.onClick.RemoveListener(()=>{
                CloseIME();
                onCloaseAction?.Invoke();
                Destroy();
            });
            backBtn.onClick.RemoveListener(() => canvasWebViewPrefab.WebView.GoBack());
            forwardBtn.onClick.RemoveListener(() => canvasWebViewPrefab.WebView.GoForward());
            refreshBtn.onClick.RemoveListener(() => canvasWebViewPrefab.WebView.Reload());
            reloadBtn.onClick.RemoveListener(() => canvasWebViewPrefab.WebView.Reload());
            canvasWebViewPrefab.WebView.LoadProgressChanged -= ProgressChanged;
            canvasWebViewPrefab.WebView.MessageEmitted -= MessageEmitted;
        }

        async void ProgressChanged (object sender, ProgressChangedEventArgs eventArgs)
        {
            if(eventArgs.Type == ProgressChangeType.Failed)
            {
                errorPanel.gameObject.SetActive(true);
            }
            if(eventArgs.Type == ProgressChangeType.Finished || eventArgs.Type == ProgressChangeType.Updated)
            {
                errorPanel.gameObject.SetActive(false);
            }
            var canGoBack = await canvasWebViewPrefab.WebView.CanGoBack();
            ChangeButtonInteractable(canGoBack, backBtn);
            var canGoForward = await canvasWebViewPrefab.WebView.CanGoForward();
            ChangeButtonInteractable(canGoForward, forwardBtn);
        }
        
        public void MessageEmitted(object sender, EventArgs<string> eventArgs)
        {
            try
            {
                // 为了兼容老版本 JS桥 closeEvent
                if(eventArgs.Value == "closeEvent")
                {
                    CloseIME();
                    Destroy();
                    return;
                }

                Command command = JsonConvert.DeserializeObject<Command>(eventArgs.Value);
                if(command.cmd == "close")
                {
                    CloseIME();
                    Destroy();
                }
                else if(command.cmd == "open_in_browser")
                {
                    var args = (OpenInBrowserArgs)command.args;
                    Application.OpenURL(args.url);
                }
                else if(command.cmd == "list")
                {
                    var obj = new JSCallback
                    {
                        cmd = "list",
                        response = new ListJSResponse
                        {
                            supportedCmds = new string[] { "close", "open_in_browser", "list", "open_short_link", "captcha_result"}
                        }
                    };
                    var jsCallback = $"window.handleComboCallback('{JsonConvert.SerializeObject(obj)}')";
                    canvasWebViewPrefab.WebView.ExecuteJavaScript(jsCallback, null);
                }
                else if(command.cmd == "open_short_link")
                {
                    var args = (OpenShortLinkArgs)command.args;
                    var obj = new JSCallback();
                    GamerManager.Instance.OpenShortLink(args.shortLink, args.gameData, result => {
                        if(result.IsSuccess)
                        {
                            obj = new JSCallback
                            {
                                cmd = "open_short_link",
                                response = new OpenShortLinkJSSuccessResponse
                                {
                                    error = ""
                                }
                            };
                        }
                        else
                        {
                            var error = ComboSDKError.Create(result.Error);
                            obj = new JSCallback
                            {
                                cmd = "open_short_link",
                                response = new OpenShortLinkJSFailResponse
                                {
                                    error = error.Error ?? "",
                                    UserMessage = error.Message ?? "",
                                    // 转为 url 编码，方便 web 解析
                                    DeveloperMessage = HttpUtils.UrlEncode(error.DetailMessage) ?? ""
                                }
                            };
                        }
                        
                        var jsCallback = $"window.handleComboCallback('{JsonConvert.SerializeObject(obj)}')";
                        canvasWebViewPrefab.WebView.ExecuteJavaScript(jsCallback, null);
                    });
                }
                else if(command.cmd == "captcha_result")
                {
                    var args = (CaptchaResultArgs)command.args;
                    ReceiveCaptchaResult.Invoke(new ReceiveCaptchaResult
                    {
                        captchaId = args.captchaId,
                        captchaResult = args.captchaResult
                    });
                    CloseIME();
                    Destroy();
                }
            }
            catch(Exception ex)
            {
                Log.E($"Invalid JSON message: {ex.Message}");
                var error = new InternalError(ex);
                ErrorTrackingManager.Instance.CaptureException(error);
            }
        }

        public void SetUrl(string url) {
            this.url = url;
        }

        public void SetShowNavigationBar(bool showNavigationBar)
        {
            this.showNavigationBar = showNavigationBar;
        }

        public void SetCloseAction(Action action)
        {
            onCloaseAction = action;
        }

        protected override IEnumerator OnShow()
        {
            yield return null;
        }
        protected override IEnumerator OnHide()
        {
            yield return null;
        }

        private void ChangeButtonInteractable(bool interactable, Button button)
        {
            button.interactable = interactable;
        }

        private void CloseIME()
        {
            var keyboardListener = FindObjectOfType<Vuplex.WebView.Internal.NativeKeyboardListener>();
            if (keyboardListener != null) {
                // 禁用组件
                keyboardListener.enabled = false;
            }
            Input.imeCompositionMode = IMECompositionMode.Off;
            UnityEngine.EventSystems.EventSystem.current.SetSelectedGameObject(null);
        }
    }
}
#endif