#if UNITY_STANDALONE
using System;
using System.Collections;
using _Core;
using Newtonsoft.Json;
using Seayoo.ComboSDK.Windows;
using UnityEngine;
using UnityEngine.UI;
using Vuplex.WebView;

namespace _Combo
{
    [ViewPrefab("Prefabs/ComboAnnouncementsView")]
    internal class WebViewV2 : View<WebViewV2>
    {
        public string url = "";
        public int width = 80;
        public int height = 80;
        public RectTransform webviewRect;
        public Button reloadBtn;
        public Text reloadText;
        public CanvasRenderer errorPanel;
        private CanvasWebViewPrefab canvasWebViewPrefab;
        private bool isSuccess = false;
        private WebViewType webViewType;
        private double resolution;
        private bool backgroundEnable;

        async void Start()
        {
            EventSystem.Register(this);

            var webView = Web.CreateWebView();
            await webView.Init(100, 100);
            webView.LoadUrl("about:blank");
            await webView.WaitForNextPageLoadToFinish();
            var userAgent = await webView.ExecuteJavaScript("navigator.userAgent");
            webView.Dispose();
            Web.SetUserAgent(userAgent + $"combo-sdk-windows/{Seayoo.ComboSDK.Windows.Version.Version.SDKVersion}");

            if(width == 0 || height == 0)
            {
                width = 80;
                height = 80;
            }
            if(width < 0 || width > 100)
            {
                width = 80;
            }
            if(height < 0 || height > 100)
            {
                height = 80;
            }
            webviewRect.localScale = new Vector2((float)width/100, (float)height/100);
            canvasWebViewPrefab = CanvasWebViewPrefab.Instantiate();
            canvasWebViewPrefab.Visible = false;
            canvasWebViewPrefab.Resolution = (float)resolution;
            UIController.Instance.ShowLoading(0.5f);
            canvasWebViewPrefab.transform.SetParent(webviewRect, false);
            reloadText.text = I18n.T("webview_reload_button");

            await canvasWebViewPrefab.WaitUntilInitialized();
            canvasWebViewPrefab.WebView.SetDefaultBackgroundEnabled(!backgroundEnable);

            canvasWebViewPrefab.WebView.MessageEmitted += MessageEmitted;
            canvasWebViewPrefab.WebView.LoadProgressChanged += ProgressChanged;
            canvasWebViewPrefab.WebView.LoadUrl(url);
            var keyboardListener = FindObjectOfType<Vuplex.WebView.Internal.NativeKeyboardListener>();
            if (keyboardListener != null) {
                // 启用组件
                keyboardListener.enabled = true;
            }
            Input.imeCompositionMode = IMECompositionMode.On;
        }

        void OnDestroy()
        {
            EventSystem.UnRegister(this);
            canvasWebViewPrefab.WebView.MessageEmitted -= MessageEmitted;
            canvasWebViewPrefab.WebView.LoadProgressChanged -= ProgressChanged;
        }

        public void ProgressChanged (object sender, ProgressChangedEventArgs eventArgs)
        {
            if(eventArgs.Type == ProgressChangeType.Failed)
            {
                UIController.Instance.HideLoading();
                errorPanel.gameObject.SetActive(true);
                LoadFailed.Invoke();
                Destroy();
            }
            else if(eventArgs.Type == ProgressChangeType.Finished)
            {
                UIController.Instance.HideLoading();
                errorPanel.gameObject.SetActive(false);
                canvasWebViewPrefab.Visible = true;
            }
        }
        
        public void MessageEmitted(object sender, EventArgs<string> eventArgs)
        {
            try
            {
                // 为了兼容老版本 JS桥 closeEvent
                if(eventArgs.Value == "closeEvent")
                {
                    CloseView();
                    return;
                }

                Command command = JsonConvert.DeserializeObject<Command>(eventArgs.Value);
                if(command.cmd == "close")
                {
                    CloseView();
                }
                else if(command.cmd == "open_in_browser")
                {
                    var args = (OpenInBrowserArgs)command.args;
                    Application.OpenURL(args.url);
                }
                else if(command.cmd == "list")
                {
                    var obj = new JSCallback
                    {
                        cmd = "list",
                        response = new ListJSResponse
                        {
                            supportedCmds = new string[] { "close", "open_in_browser", "list", "open_short_link", "captcha_result"}
                        }
                    };
                    var jsCallback = $"window.handleComboCallback('{JsonConvert.SerializeObject(obj)}')";
                    canvasWebViewPrefab.WebView.ExecuteJavaScript(jsCallback, null);
                }
                else if(command.cmd == "open_short_link")
                {
                    var args = (OpenShortLinkArgs)command.args;
                    var obj = new JSCallback();
                    GamerManager.Instance.OpenShortLink(args.shortLink, args.gameData, result => {
                        if(result.IsSuccess)
                        {
                            obj = new JSCallback
                            {
                                cmd = "open_short_link",
                                response = new OpenShortLinkJSSuccessResponse
                                {
                                    error = ""
                                }
                            };
                        }
                        else
                        {
                            var error = ComboSDKError.Create(result.Error);
                            obj = new JSCallback
                            {
                                cmd = "open_short_link",
                                response = new OpenShortLinkJSFailResponse
                                {
                                    error = error.Error ?? "",
                                    UserMessage = error.Message ?? "",
                                    // 转为 url 编码，方便 web 解析
                                    DeveloperMessage = HttpUtils.UrlEncode(error.DetailMessage) ?? ""
                                }
                            };
                        }
                        var jsCallback = $"window.handleComboCallback('{JsonConvert.SerializeObject(obj)}')";
                        canvasWebViewPrefab.WebView.ExecuteJavaScript(jsCallback, null);
                    }); 
                }
                else if(command.cmd == "captcha_result")
                {
                    var args = (CaptchaResultArgs)command.args;
                    ReceiveCaptchaResult.Invoke(new ReceiveCaptchaResult
                    {
                        webViewType = webViewType,
                        captchaId = args.captchaId,
                        captchaResult = args.captchaResult
                    });
                    CloseIME();
                    Destroy();
                }
            }
            catch(Exception ex)
            {
                Log.E($"Invalid JSON message: {ex.Message}");
                var error = new InternalError(ex);
                ErrorTrackingManager.Instance.CaptureException(error);
            }
        }

        public void SetUrl(string url) {
            this.url = url;
        }

        public void SetWidth(int width)
        {
            this.width = width;
        }

        public void SetHeight(int height)
        {
            this.height = height;
        }

        public void SetType(WebViewType webViewType)
        {
            this.webViewType = webViewType;
        }

        public void SetResolution(double resolution)
        {
            this.resolution = resolution;
        }

        public void SetDefaultBackgroundEnabled(bool enable)
        {
            backgroundEnable = enable;
        }

        protected override IEnumerator OnShow()
        {
            yield return null;
        }
        protected override IEnumerator OnHide()
        {
            yield return null;
        }

        private void CloseIME()
        {
            var keyboardListener = FindObjectOfType<Vuplex.WebView.Internal.NativeKeyboardListener>();
            if (keyboardListener != null) {
                // 禁用组件
                keyboardListener.enabled = false;
            }
            Input.imeCompositionMode = IMECompositionMode.Off;
            UnityEngine.EventSystems.EventSystem.current.SetSelectedGameObject(null);
        }

        private void CloseView()
        {
            if(isSuccess)
            {
                return;
            }
            isSuccess = true;
            CloseIME();
            LoadSuccess.Invoke(new LoadSuccess { webViewType = webViewType});
            Destroy();
        }
    }
}
#endif