using System;
using System.IO;
using System.Net;
using System.Text;
using UnityEngine;

#if UNITY_EDITOR
using UnityEditor;
#endif

namespace _Core
{
    public class HttpServer : MonoBehaviour
    {
        private static HttpServer _instance;
        public static HttpServer Instance 
        {
            get 
            {
                if (_instance == null)
                {
                    _instance = FindObjectOfType<HttpServer>();
                    if (_instance == null)
                    {
                        GameObject go = new GameObject("HttpServer");
                        _instance = go.AddComponent<HttpServer>();
                    }
                }
                return _instance;
            }
        }

        private HttpListener listener;
        private string localIp = "http://127.0.0.1:3888/";
        private string contentFilePath;

        private void Awake()
        {
            if (_instance == null)
            {
                _instance = this;
                DontDestroyOnLoad(gameObject); // 保证实例在场景切换时不被销毁
            }
            else if (_instance != this)
            {
                Destroy(gameObject); // 保证单例唯一性
            }
        }

        private void OnEnable()
        {
            Application.quitting += OnApplicationQuit;
#if UNITY_EDITOR
            EditorApplication.playModeStateChanged += OnPlayModeStateChanged;
#endif
        }

        private void OnDisable()
        {
            Application.quitting -= OnApplicationQuit;
#if UNITY_EDITOR
            EditorApplication.playModeStateChanged -= OnPlayModeStateChanged;
#endif
        }

        public void StartServer()
        {
            if (listener != null && listener.IsListening)
            {
                Debug.Log("HttpServer is running.");
                return; // 避免重复启动
            }

            contentFilePath = GetPath();
            try 
            {
                listener = new HttpListener();
                listener.AuthenticationSchemes =  AuthenticationSchemes.Anonymous;
                listener.Prefixes.Add(localIp);
                listener.Start();
                listener.BeginGetContext(Response, null);
            }
            catch (HttpListenerException ex)
            {
                Log.E($"Failed to start server: {ex.Message}");
            }
            catch (Exception ex)
            {
                Log.E($"Critical error: {ex.Message}");
            }
            Log.I($"Server Start: {DateTime.Now}, address: {localIp}");
        }

        private string GetPath()
        {
            string projectRoot = Application.streamingAssetsPath;

            // 拼接相对路径
            string relativePath = "ComboSDK";
            string fullPath = Path.Combine(projectRoot, relativePath);
            fullPath = fullPath.Replace("\\", "/");
            return fullPath;
        }

        private void Response(IAsyncResult ar)
        {
            try
            {
                if (listener == null || !listener.IsListening)
                    return;

                listener.BeginGetContext(Response, null);
                Debug.Log($"{DateTime.Now} receive new request");

                var context = listener.EndGetContext(ar);
                var request = context.Request;
                var response = context.Response;

                string filename = context.Request.Url.AbsolutePath.Trim('/');
                if (string.IsNullOrEmpty(filename))
                {
                    context.Response.StatusCode = (int)HttpStatusCode.BadRequest;
                    byte[] errorMsg = Encoding.UTF8.GetBytes("Invalid request");
                    context.Response.OutputStream.Write(errorMsg, 0, errorMsg.Length);
                    context.Response.OutputStream.Close();
                    return;
                }

                filename = Path.Combine(contentFilePath, filename);
                filename = filename.Replace("\\", "/");
                if (!File.Exists(filename))
                {
                    context.Response.StatusCode = (int)HttpStatusCode.NotFound;
                    byte[] errorMsg = Encoding.UTF8.GetBytes("File not found");
                    context.Response.OutputStream.Write(errorMsg, 0, errorMsg.Length);
                    context.Response.OutputStream.Close();
                    return;
                }

                byte[] fontBytes = File.ReadAllBytes(filename);

                // 设置响应头
                context.Response.StatusCode = (int)HttpStatusCode.OK;
                context.Response.ContentLength64 = fontBytes.Length;

                // 添加CORS头
                context.Response.Headers.Add("Access-Control-Allow-Origin", "*");

                // 设置缓存控制（可选）
                context.Response.Headers.Add("Cache-Control", "no-cache");

                context.Response.OutputStream.Write(fontBytes, 0, fontBytes.Length);
                context.Response.OutputStream.Close();
            }
            catch (Exception ex)
            {
                Debug.LogError($"An error occurred while processing the request: {ex.Message}");
            }
        }

        public void StopServer()
        {
            if (listener != null && listener.IsListening)
            {
                listener.Stop();
                listener.Close();
                listener = null; // 释放资源
                Debug.Log($"Server stop: {DateTime.Now}");
            }
        }

        private void OnApplicationQuit()
        {
            StopServer();
        }

#if UNITY_EDITOR
        private void OnPlayModeStateChanged(PlayModeStateChange state)
        {
            if (state == PlayModeStateChange.ExitingPlayMode)
            {
                StopServer();
            }
        }
#endif
    }
}
